 #version 330 core
layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;

in vec2 TexCoords;
in vec3 TangentNormal;
in vec3 FragPos;
in vec3 TangentLightPos;
in vec3 TangentViewPos;
in vec3 TangentFragPos;

uniform samplerCubeShadow depthMap;
uniform sampler2D texture_diffuse;
uniform sampler2D texture_normal;

uniform bool useNormalMapping;

uniform vec3 lightColor;
uniform vec3 lightPos; 
uniform vec3 viewPos;

uniform float farPlane;



//returns 1 if fragment is shadow, 0 when it is not in shadow
float PointShadowCalculation(vec3 fragPos){

	//retrieve depth of the cubemap. Direction is the direction from fragPos to lightPos
	vec3 fragToLight = fragPos - lightPos ;
	//Get normalized depth value between light source and closest visible fragment
	float currentDepth = (length(fragToLight)-0.03) / farPlane;
	float closestDepth = texture(depthMap, vec4(fragPos - lightPos, currentDepth));
	return -closestDepth;

}


void main()
{   
	vec3 normal = normalize(TangentNormal);
	if(useNormalMapping){
		//Get normal out of the normal map in range [0.1]
		normal = texture(texture_normal, TexCoords).rgb;
		// Transform normal vector to range [-1,1]
		normal = normalize(normal * 2.0 - 1.0);	//This normal is in tangent space
	}
	//Perform phong shading
    vec4 objectColor = texture(texture_diffuse, TexCoords);
	//Add ambient lighting
    float ambientStrength = 0.1;
    vec3 ambient = ambientStrength * lightColor;

	//add diffuse lighting
	vec3 lightDir = normalize(TangentLightPos - TangentFragPos); 
	float diff = max(dot(normal, lightDir), 0.0);
	vec3 diffuse = diff * lightColor;

	//add specular lighting
    float specularStrength = 0.2;
	float shininess = 30;
	vec3 viewDir = normalize(TangentViewPos - TangentFragPos);
	vec3 reflectDir = reflect(-lightDir, normal);  
	vec3 halfwayDir = normalize(lightDir + viewDir);  
	float spec = pow(max(dot(viewDir, halfwayDir), 0.0), shininess);
	vec3 specular = specularStrength * spec * lightColor;  
	
	//Calculate shadow
	float shadow = PointShadowCalculation(FragPos);
	vec4 result = vec4((ambient + (1.0 - shadow) * (diffuse + specular)),1.0) * objectColor;
	FragColor = result;
	//FragColor = vec4(vec3(shadow), 1.0); 
	BrightColor = vec4(0.0,0.0,0.0,1.0);
}