 #version 330 core
layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;

in vec2 TexCoords;
in vec3 Normal;
in vec3 FragPos;

uniform samplerCubeShadow depthMap;
uniform sampler2D texture_diffuse;

uniform vec3 lightColor;
uniform vec3 lightPos; 
uniform vec3 viewPos;

uniform float farPlane;


// array of offset direction for sampling for PCF algorithm
vec3 sampleOffsetDirections[20] = vec3[]
(
   vec3(1, 1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1, 1,  1), 
   vec3(1, 1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1, 1,  0),
   vec3(1, 0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1, 0, -1),
   vec3(0, 1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0, 1, -1)
);

float VectorToDepth (vec3 Vec)
{
    vec3 AbsVec = abs(Vec);
    float LocalZcomp = max(AbsVec.x, max(AbsVec.y, AbsVec.z));

    // Replace f and n with the far and near plane values you used when
    //   you drew your cube map.
    float f = 0.1;
    float n = 100.0;

    float NormZComp = (f+n) / (f-n) - (2*f*n)/(f-n)/LocalZcomp;
    return (NormZComp + 1.0) * 0.5;
}

//returns 1 if fragment is shadow, 0 when it is not in shadow
float PointShadowCalculation(vec3 fragPos){

	//retrieve depth of the cubemap. Direction is the direction from fragPos to lightPos
	vec3 fragToLight = fragPos - lightPos ;
	//Get normalized depth value between light source and closest visible fragment
	float currentDepth = (length(fragToLight)+0.2) / farPlane;
	float closestDepth = texture(depthMap, vec4(fragPos - lightPos, currentDepth));
	return -closestDepth;
	//return closestDepth;
	//closestDepth is in range [0,1] - transform it back to [0,farPlane]
	//	closestDepth = closestDepth * farPlane;

	//return closestDepth;
/*	
	//compare currentDepth and closestDepth
	//Apply PCF algorithm so that the shadow appears smoother
	float shadow = 0.0;
	float bias = 0.15;
	int samples = 20;	//sample in 20 different directions
	float viewDistance = length(viewPos - fragPos);
	//make radius dependent on the distance to the viewer
	float radius = (1.0 + (viewDistance / farPlane)) / 50.0 ;
	for(int i = 0; i < samples; ++i)
	{
		float closestDepth = texture(depthMap, fragToLight + sampleOffsetDirections[i]* radius).r;
		closestDepth *= farPlane; //Undo Mapping [0;1]
		if(currentDepth - bias > closestDepth){
			shadow += 1.0;
		}
	}
	shadow = shadow / float(samples);
	return shadow;
	*/
}


void main()
{   
	//Perform phong shading
    vec4 objectColor = texture(texture_diffuse, TexCoords);
	//Add ambient lighting
    float ambientStrength = 0.2;
    vec3 ambient = ambientStrength * lightColor;

	//add diffuse lighting
	vec3 normal = normalize(Normal);
	vec3 lightDir = normalize(lightPos - FragPos); 
	float diff = max(dot(normal, lightDir), 0.0);
	vec3 diffuse = diff * lightColor;

	//add specular lighting
    float specularStrength = 0.5;
	float shininess = 64;
	vec3 viewDir = normalize(viewPos - FragPos);
	vec3 reflectDir = reflect(-lightDir, normal);  
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
	vec3 specular = specularStrength * spec * lightColor;  
	/*
	//Calculate shadow
	float shadow = PointShadowCalculation(FragPos);
	shadow = 0;
	vec4 result = vec4((ambient + (1.0 - shadow) * (diffuse + specular)),1.0) * objectColor;
	FragColor = result;
	//FragColor = vec4(vec3(shadow), 1.0); 
	*/
	
	 vec3 result = (ambient + diffuse + specular) * objectColor.xyz;
    FragColor = vec4(result, 1.0);
	
	BrightColor = vec4(0.0,0.0,0.0,1.0);
}