 #version 330 core
out vec4 FragColor;

in vec2 TexCoords;
in vec3 Normal;
in vec3 FragPos;

uniform sampler2D texture_diffuse1;
uniform vec3 lightColor;
uniform vec3 lightPos; 
uniform vec3 viewPos;

void main()
{   
	//Perform phong shading
    vec4 objectColor = texture(texture_diffuse1, TexCoords);

	//Add ambient lighting
    float ambientStrength = 0.3;
    vec3 ambient = ambientStrength * lightColor;

	//add diffuse lighting
	vec3 norm = normalize(Normal);
	vec3 lightDir = normalize(lightPos - FragPos); 
	float diff = max(dot(norm, lightDir), 0.0);
	vec3 diffuse = diff * lightColor;

	//add specular lighting
    float specularStrength = 0.5;
	float shininess = 64;
	vec3 viewDir = normalize(viewPos - FragPos);
	vec3 reflectDir = reflect(-lightDir, norm);  
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), shininess);
	vec3 specular = specularStrength * spec * lightColor;  
	vec4 result = vec4((specular + ambient + diffuse),1.0) * objectColor;
	FragColor = result;
	
}