#version 330 core
layout (location = 0) in vec3 aPos;
layout (location = 1) in vec3 aNormal;
layout (location = 2) in vec2 aTexCoords;
layout (location = 3) in vec3 aTangent;
layout (location = 4) in vec3 aBitangent;

out vec2 TexCoords;
out vec3 TangentNormal;	//In tangent space
out vec3 FragPos;
out vec4 PostPerspectiveFragPos;

//Positions in tangent space only used if normal mapping is used
out vec3 TangentLightPos;	
out vec3 TangentViewPos;
out vec3 TangentFragPos;

uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;

uniform bool useNormalMapping;
uniform vec3 lightPos;
uniform vec3 viewPos;

void main()
{
    FragPos = vec3(model * vec4(aPos, 1.0));
	TexCoords = aTexCoords;  
	
	//Compute the LightPos, FragPos and ViewPos in Tangent Space. 
	mat3 normalMatrix = transpose(inverse(mat3(model)));
	vec3 T = normalize(normalMatrix * aTangent);
	vec3 N = normalize(normalMatrix * aNormal);
	T = normalize(T - dot(T,N)*N);
	vec3 B = cross(N,T);

	mat3 TBN = transpose(mat3(T,B,N));
	TangentLightPos = TBN * lightPos;
	TangentFragPos = TBN * FragPos;
	TangentViewPos = TBN * viewPos;
	TangentNormal = TBN * aNormal;	
		//Normal = mat3(transpose(inverse(model))) * aNormal;  //Corrects the normal vector for non-uniform scaling. Uncommend if non-uniform scaling is performed
	
	gl_Position = projection * view * model * vec4(aPos, 1.0);
	PostPerspectiveFragPos = gl_Position;
}